"use strict";

exports.__esModule = true;
exports.default = void 0;

var _fs = require("fs");

var _path = _interopRequireDefault(require("path"));

var _mdx = _interopRequireDefault(require("@mdx-js/mdx"));

var _fileEntryCache = _interopRequireDefault(require("file-entry-cache"));

var _uniqBy = _interopRequireDefault(require("lodash/uniqBy"));

var reactDocgen = _interopRequireWildcard(require("react-docgen"));

var _tinyCase = require("tiny-case");

var _jsDocHandler = _interopRequireDefault(require("./jsDocHandler"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-await-in-loop */

/* eslint-disable no-param-reassign */
// @ts-ignore
// @ts-ignore
// @ts-ignore
// @ts-ignore
const noop = `() => null`;
const PLUGIN = 'docusaurus-plugin-react-metadata';

var _default = (ctx, options = {}) => {
  const {
    src,
    mdx: mdxOptions,
    docgen = {},
    babel = {},
    watchPaths,
    parserOptions
  } = options;
  const defaultHandlers = [...reactDocgen.defaultHandlers, _jsDocHandler.default];
  const dataDir = `${ctx.generatedFilesDir}/${PLUGIN}/${options.id}/`;

  const cache = _fileEntryCache.default.create('timestamps', dataDir);

  const extraCache = watchPaths ? _fileEntryCache.default.create('extra-paths', dataDir) : null;

  async function stringify(value, write, parent) {
    let str = '';

    if (Array.isArray(value)) {
      const values = await Promise.all(value.map(v => stringify(v, write, (v == null ? void 0 : v.name) || parent)));
      str = `[${values.join(',\n')}]`;
    } else if (value && typeof value === 'object') {
      str += '{\n';

      for (const [key, keyValue] of Object.entries(value)) {
        if (key === 'description') {
          if (keyValue) {
            const file = await write(parent, await (0, _mdx.default)(keyValue, mdxOptions));
            str += `"${key}": require('./${file}').default,\n`;
          } else {
            str += `"${key}": ${noop},\n`;
          }
        } else {
          str += `"${key}": ${await stringify(keyValue, write, key)},\n`;
        }
      }

      str += '}';
    } else {
      str += JSON.stringify(value);
    }

    return str;
  }

  return {
    name: PLUGIN,

    getPathsToWatch() {
      return [].concat(src, watchPaths || []);
    },

    getThemePath() {
      return _path.default.join(__dirname, '..', 'lib', 'theme');
    },

    getTypeScriptThemePath() {
      return _path.default.resolve(__dirname, './theme');
    },

    async loadContent() {
      const {
        globby
      } = await import('globby');
      let rerunAll = false;
      const [allFiles, extraFiles] = await Promise.all([globby(src), extraCache && globby(watchPaths)]);

      if (extraCache) {
        rerunAll = !!extraCache.getUpdatedFiles(extraFiles).length;
      }

      const changedFiles = cache.getUpdatedFiles(allFiles);
      const files = rerunAll ? allFiles : changedFiles;
      const content = await Promise.all(files.map(async file => {
        try {
          return {
            docgen: reactDocgen.parse(await _fs.promises.readFile(file), docgen.resolver || reactDocgen.resolver.findAllComponentDefinitions, defaultHandlers.concat(docgen.handlers || []), {
              filename: file,
              parserOptions,
              ...babel
            }),
            file
          };
        } catch (e) {
          if (e.message.includes('No suitable component definition found')) return null;
          console.error(e.message, file);
        }

        return null;
      }));
      return (0, _uniqBy.default)(content.filter(Boolean), c => c.file);
    },

    async contentLoaded({
      content,
      actions
    }) {
      const {
        createData
      } = actions;
      await Promise.all( // eslint-disable-next-line @typescript-eslint/no-shadow
      content.flatMap(({
        file,
        docgen
      }) => {
        return docgen.map(async component => {
          component.displayName = component.displayName || (0, _tinyCase.pascalCase)(_path.default.basename(file, _path.default.extname(file)));
          const name = component.displayName;
          component.props = Object.entries(component.props || {}).map( // eslint-disable-next-line @typescript-eslint/no-shadow
          ([name, value]) => ({
            name,
            ...value
          }));

          const write = async (p = '', data) => {
            const filename = `${name}_${p ? `${p}_` : ''}description.js`;
            await createData(filename, `import { mdx } from '@mdx-js/react';\n\n${data}`);
            return filename;
          };

          return createData(`${name}.js`, `module.exports = ${await stringify(component, write)}`);
        });
      }));
      cache.reconcile(true);
      extraCache == null ? void 0 : extraCache.reconcile(true);
    },

    configureWebpack() {
      const scope = options.id === 'default' ? '' : `/${options.id}`;
      return {
        resolve: {
          alias: {
            [`@react-metadata${scope}`]: dataDir
          }
        }
      };
    }

  };
};

exports.default = _default;