"use strict";

exports.__esModule = true;
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _TypescriptTypeValue = _interopRequireDefault(require("./TypescriptTypeValue"));

var _utils = require("../utils");

var _doclets = require("../doclets");

var _jsxRuntime = require("react/jsx-runtime");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function mapToTypes(propType, tags) {
  const map = pt => mapToTypes(pt, tags);

  const name = (0, _utils.getDisplayTypeName)(propType.name);
  const docletType = (0, _utils.getDoclet)(tags, 'type');

  switch (propType.name) {
    case 'object':
      return {
        name: 'Object'
      };

    case 'shape':
      return {
        name: 'signature',
        type: 'object',
        raw: '',
        signature: {
          properties: Object.entries(propType.value).map(([key, value]) => ({
            key,
            value: { ...map(value),
              required: value.required
            }
          }))
        }
      };

    case 'union':
      return {
        name: 'union',
        elements: propType.value.map(n => typeof n === 'string' ? {
          name: n
        } : n).map(map)
      };

    case 'enum':
      {
        const enumValues = propType.value || [];

        if (!Array.isArray(enumValues)) {
          return {
            name: enumValues
          };
        }

        return {
          name: 'union',
          elements: enumValues.map(e => ({
            name: e.value
          }))
        };
      }

    case 'arrayOf':
      return {
        name: 'Array',
        elements: [map(propType.value)]
      };

    case 'custom':
      return {
        name: (0, _doclets.cleanDocletValue)(docletType || propType.raw)
      };

    default:
      return {
        name: 'literal',
        value: name
      };
  }
}

function PropTypeValue({
  type,
  tags,
  tokens
}) {
  const tsType = (0, _react.useMemo)(() => mapToTypes(type, tags), [type, tags]);
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_TypescriptTypeValue.default, {
    type: tsType,
    tags: tags,
    tokens: tokens
  });
}

var _default = PropTypeValue;
exports.default = _default;