"use strict";

exports.__esModule = true;
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _utils = require("../utils");

var _jsxRuntime = require("react/jsx-runtime");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isPrimitive = name => name[0].toLowerCase() === name[0];

const isArray = t => t.name === 'Array';

const isComplex = t => ['signature', 'union', 'tuple', 'intersect'].includes(t.name);

function getType(type, tags, tokens) {
  const concrete = type;

  const get = t => getType(t, tags, tokens);

  const t = token => {
    var _tokens$token;

    return (_tokens$token = tokens == null ? void 0 : tokens[token]) != null ? _tokens$token : `pt-token pt-${token}`;
  };

  function renderName(name) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
      className: t(isPrimitive(name) ? 'primitive' : 'name'),
      children: name
    });
  }

  function renderNamedType(namedType) {
    return /*#__PURE__*/(0, _jsxRuntime.jsxs)("span", {
      className: t('type_signature'),
      children: [renderName(namedType.name), 'elements' in namedType && /*#__PURE__*/(0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
        children: ['<', /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
          className: t('elements'),
          children: (0, _utils.joinElements)(namedType.elements, ', ', get)
        }), '>']
      })]
    });
  }

  function renderArray(arrType) {
    return isComplex(arrType.elements[0]) ? renderNamedType(arrType) : /*#__PURE__*/(0, _jsxRuntime.jsxs)("span", {
      className: t('array'),
      children: [get(arrType.elements[0]), "[]"]
    });
  }

  const userType = (0, _utils.getDoclet)(tags, 'type');
  if (userType) return renderName(userType);

  switch (concrete.name) {
    case 'literal':
      return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
        className: t('literal'),
        children: concrete.value
      });

    case 'signature':
      return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
        className: t(`${concrete.type}-signature`),
        children: concrete.type === 'function' ? /*#__PURE__*/(0, _jsxRuntime.jsxs)(_jsxRuntime.Fragment, {
          children: ["(", (0, _utils.joinElements)(concrete.signature.arguments, ', ', arg => /*#__PURE__*/(0, _jsxRuntime.jsxs)("span", {
            className: t('argument'),
            children: [arg.name, ": ", get(arg.type)]
          })), ') => ', /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
            className: t('return'),
            children: get(concrete.signature.return)
          })]
        }) : concrete.raw
      });

    case 'union':
      return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
        className: t('union'),
        children: (0, _utils.joinElements)(concrete.elements, null, get)
      });

    case 'tuple':
      {
        return /*#__PURE__*/(0, _jsxRuntime.jsxs)("span", {
          className: t('tuple'),
          children: ["[", (0, _utils.joinElements)(concrete.elements, ', ', get), "]"]
        });
      }

    case 'intersect':
      return /*#__PURE__*/(0, _jsxRuntime.jsx)("span", {
        className: t('intersect'),
        children: (0, _utils.joinElements)(concrete.elements, null, get)
      });

    default:
      if (isArray(type)) return renderArray(type);
      if (isPrimitive(type.name)) renderName(type.name);
      return renderNamedType(type);
  }
}

function TypescriptTypeValue({
  type,
  tags,
  tokens
}) {
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_jsxRuntime.Fragment, {
    children: getType(type, tags, tokens)
  });
}

var _default = TypescriptTypeValue;
exports.default = _default;