"use strict";

exports.__esModule = true;
exports.default = renderProps;

var _sortBy = _interopRequireDefault(require("lodash/sortBy"));

var _react = _interopRequireDefault(require("react"));

var _utils = require("../utils");

var _DefaultPropValue = _interopRequireDefault(require("./DefaultPropValue"));

var _PropTypeValue = _interopRequireDefault(require("./PropTypeValue"));

var _TypescriptTypeValue = _interopRequireDefault(require("./TypescriptTypeValue"));

var _jsxRuntime = require("react/jsx-runtime");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isElementType = (name, types) => {
  return types.some(t => {
    if (typeof t === 'string') return t === name;
    return !!(name != null && name.match(t));
  });
};

function renderProps(propsData, {
  tokenMap,
  elementTypes = ['elementType', /React\.ComponentType(<.*>)?/]
} = {}) {
  return (0, _sortBy.default)(propsData, 'name').filter(prop => {
    var _prop$tags;

    return !((_prop$tags = prop.tags) != null && _prop$tags.find(d => d.name === 'private' || d.name === 'ignore'));
  }).map(propData => {
    const {
      name,
      type,
      defaultValue,
      description,
      tags,
      tsType
    } = propData;
    const docletMap = (0, _utils.docletsToMap)(tags);
    const typeName = (0, _utils.getTypeName)(propData);
    const descJsx = typeof description === 'function' ? /*#__PURE__*/_react.default.createElement(description) : /*#__PURE__*/(0, _jsxRuntime.jsx)("p", {
      children: description || ''
    });
    let renderedType = null;
    if (tsType) renderedType = /*#__PURE__*/(0, _jsxRuntime.jsx)(_TypescriptTypeValue.default, {
      type: tsType,
      tags: tags,
      tokens: tokenMap
    });else if (type) renderedType = /*#__PURE__*/(0, _jsxRuntime.jsx)(_PropTypeValue.default, {
      type: type,
      tags: tags,
      tokens: tokenMap
    });
    return {
      name,
      tags,
      typeName,
      description: descJsx,
      deprecated: docletMap.get('deprecated'),
      type: renderedType,
      defaultValue: defaultValue && /*#__PURE__*/(0, _jsxRuntime.jsx)(_DefaultPropValue.default, { ...defaultValue,
        isElementType: isElementType(typeName, elementTypes)
      }),
      propData
    };
  });
}